package org.netkernel.xml.saxon.accessor.util;

import org.apache.xml.resolver.Catalog;
import org.apache.xml.resolver.CatalogManager;
import org.netkernel.container.IKernel;
import org.netkernel.layer0.nkf.INKFRequest;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.NKFException;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.urii.SimpleIdentifierImpl;
import org.netkernel.layer0.util.NKProtocolHandlerRegistry;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.netkernel.request.IRequestScopeLevel;
import org.netkernel.request.impl.RequestScopeLevelImpl;
import org.netkernel.urii.IIdentifier;
import org.netkernel.urii.ISpace;
import org.netkernel.urii.impl.Version;

public class CatalogConfigAccessor extends StandardAccessorImpl
{
	public CatalogConfigAccessor()
	{	this.declareThreadSafe();
	}
	
	@Override
	public void onSource(INKFRequestContext context) throws Exception
	{
		CatalogManager cm= new CatalogManager();
		cm.setRelativeCatalogs(false);
		
		Catalog catalog= new Catalog(cm);
		catalog.setupReaders();
		
		IHDSNode importedSpaces=context.source("active:SimpleImportDiscovery", IHDSNode.class);
		IKernel k=context.getKernelContext().getKernel();
		
		for (IHDSNode spaceNode : importedSpaces.getNodes("/config/space"))
		{
			IIdentifier uri = new SimpleIdentifierImpl(spaceNode.getFirstValue(
					"uri").toString());
			Version version = new Version(spaceNode.getFirstValue("version")
					.toString());
			ISpace space = k.getSpace(uri, version, version);
			IRequestScopeLevel scope = RequestScopeLevelImpl
					.createOrphanedRootScopeLevel(space, context
							.getKernelContext().getRequestScope());
			INKFRequest req = context
					.createRequest("res:/etc/system/XMLCatalog.xml");
			req.setRequestScope(scope);
			
			try
			{
				context.logRaw(INKFRequestContext.LEVEL_INFO, "Loading catalogs from " + uri.toString());
				IHDSNode configNode;
				Object rep = context.issueRequest(req);
				if (!(rep instanceof IHDSNode))
				{
					configNode = context.transrept(rep, IHDSNode.class);
				} else
				{
					configNode = (IHDSNode) rep;
				}
				
				for (IHDSNode catalogNode : configNode.getNodes("/XMLCatalog/catalog"))
				{
					context.logRaw(INKFRequestContext.LEVEL_FINE, "Loading catalog " + (String)catalogNode.getValue());
					catalog.parseCatalog(NKProtocolHandlerRegistry.getGlobalURL(context, scope, (String)catalogNode.getValue()));
				}
				
			} catch (NKFException e)
			{
				e.printStackTrace();
			}
		}
		context.createResponseFrom(catalog);
	}
}
