/******************************************************************************
  (c) Copyright 2009, 1060 Research Ltd                                        

  This Software is licensed to You, the licensee, for use under the terms of   
  the 1060 Public License v1.0. Please read and agree to the 1060 Public       
  License v1.0 [www.1060research.com/license] before using or redistributing   
  this software.                                                               

  In summary the 1060 Public license has the following conditions.             
  A. You may use the Software free of charge provided you agree to the terms   
  laid out in the 1060 Public License v1.0                                     
  B. You are only permitted to use the Software with components or applications
  that provide you with OSI Certified Open Source Code [www.opensource.org], or
  for which licensing has been approved by 1060 Research Limited.              
  You may write your own software for execution by this Software provided any  
  distribution of your software with this Software complies with terms set out 
  in section 2 of the 1060 Public License v1.0                                 
  C. You may redistribute the Software provided you comply with the terms of   
  the 1060 Public License v1.0 and that no warranty is implied or given.       
  D. If you find you are unable to comply with this license you may seek to    
  obtain an alternative license from 1060 Research Limited by contacting       
  license@1060research.com or by visiting www.1060research.com                 

  NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC  
  LICENSE V1.0 FOR DETAILS                                                     

  THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ     
  THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license      

  $Id$
 *****************************************************************************/
package org.netkernel.xml.saxon.accessor.xslt2;

import javax.xml.transform.Source;
import javax.xml.transform.sax.SAXSource;

import net.sf.saxon.s9api.Processor;
import net.sf.saxon.s9api.XsltCompiler;
import net.sf.saxon.s9api.XsltExecutable;

import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.representation.IReadableBinaryStreamRepresentation;
import org.netkernel.module.standard.endpoint.StandardTransreptorImpl;
import org.netkernel.xml.saxon.util.SaxonUtil;
import org.netkernel.xml.saxon.util.LocalURIResolver;
import org.netkernel.xml.util.NKFEntityResolver;
import org.netkernel.xml.util.NKFURIResolver;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;

/**
 * Transrepts an XSLT source document into a XsltExecutable using Saxon 9.
 * 
 * @author Chris Cormack
 * @author Menzo Windhouwer
 */
public class XSLTTransreptor extends StandardTransreptorImpl
{

	public XSLTTransreptor()
	{	this.declareThreadSafe();
		this.declareToRepresentation(XsltExecutable.class);
	}

	@Override
	public void onTransrept(INKFRequestContext context) throws Exception
	{	IReadableBinaryStreamRepresentation from = context
				.sourcePrimary(IReadableBinaryStreamRepresentation.class);
		Processor processor = SaxonUtil.getProcessor();

		XsltCompiler compiler = processor.newXsltCompiler();
		compiler.setURIResolver(new LocalURIResolver(context));

		XMLReader reader = XMLReaderFactory.createXMLReader();
		reader.setEntityResolver(new NKFEntityResolver(context));
		reader.setFeature("http://xml.org/sax/features/resolve-dtd-uris",
						false);
		
		InputSource src = new InputSource(from.getInputStream());
		src.setSystemId(context.getThisRequest().getIdentifier());
		Source source = new SAXSource(reader, src);

		context.createResponseFrom(compiler.compile(source));
	}
}